#include "get_util.h"
#include <ea/ssga/ssga.h>
#include <localsearch/solis.h>
#include <localsearch/cmaes.h>
#include <localsearch/simplex.h>
#include <localsearch/solisn.h>
#include <localsearch/solisn2.h>
#include <localsearch/cmaeshan.h>
#include <localsearch/mts1.h>
#include <localsearch/mts2.h>
#include <iostream>
#include <cassert>
#include <cstdio>
#include <sstream>

double string_to_double( const std::string& s )
{
   std::istringstream i(s);
   double x;
   if (!(i >> x))
     return 0;
   return x;
} 


using namespace realea;
using std::auto_ptr;

bool debug=false;

void set_InitVerbose(void) {
    debug = true;
}

bool getCrossFactor(string name, double *pfactor) {
    int num;
    
    num = sscanf(name.c_str(), "ssgac-%lf", pfactor);
    return (num >= 1);
}

bool find_str(const string &str, const string substr) {
    return (str.find(substr)!=string::npos);
}

ICrossBinary *get_Cross(string crossover) {
    ICrossBinary *cross=NULL;
    char c_alpha[20], c_pr[20];
    double alpha,pr;
    int num;

 
    if (find_str(crossover, "pblx")) { 

       if (find_str(crossover, "-") ) {
	    sscanf(crossover.c_str(), "pblx-%s", c_alpha);
	    alpha = string_to_double(c_alpha);
       }
       else {
	    alpha=0.5;
       }

       alpha = fabs(alpha);

       cross = new CrossPBLX(alpha);
    }
    else if (find_str(crossover, "blx")) {
       num = sscanf(crossover.c_str(), "blx-%s", c_alpha);
       alpha = string_to_double(c_alpha);

	if (num == 1) {
	   cross = new CrossBLX(alpha);
	}
    }

    return cross;
}

string get_EANames(string sep) {
    return "chc" + sep + "de" + sep + "ssga" + sep +"pso";
}

IEA *get_EA(string alg, Random &random) {
	string print_str;
	ICrossBinary *cross=NULL;
	string cross_str;
	IEA *ea;

	print_str = "EA:: ";

	string::size_type pos_cross = alg.find("_");

	if (pos_cross != string::npos) {
	   cross_str = alg.substr(pos_cross+1);
	   alg = alg.substr(0, pos_cross);
	}
	else {
	    cross_str = "blx-0.5";
	}

	cross = get_Cross(cross_str);

	if (cross == NULL) {
	   throw string("Crossover " +alg +":" + cross_str +" is unknown");
	}
	if (alg == "ssga") {
	   SSGA *ssga = new SSGA(&random);
	   ssga->setCross(cross);
	   ssga->setMutation(new MutationBGA());
	   ssga->setSelect(new SelectNAM(3));
	   ssga->setReplacement(new ReplaceWorst());
	   print_str += "SSGA with " +cross_str +" with NAM-3 and RW";
	   ea = ssga;	
	}
	else {
	    delete cross;
	    throw string("EA '" +alg +"' is unknown");
	}
	
	if (debug) {
	    printf("%s\n", print_str.c_str());
	}

	return ea;
}

string get_LSNames(string sep) {
    return "cmaes" + sep + "sw" + sep + "ssw" + sep + "simplex" + sep + "mts1" + sep +"mts2";
}


ILocalSearch *get_LS(string arg_ls, DomainRealPtr domain, Random *random) {
    ILocalSearch *ls;
    string print_str = "LS: ";
	
	if (arg_ls == "sw") {
	   SolisWets *sw = new SolisWets();
	   sw->setDelta(0.2);
	   print_str += "Solis Wets\n";
	   ls = sw;
	}
	else if (arg_ls.find("ssw")==0) {
	   SWNDim *sw = new SWNDim();
	   string c_strategy = arg_ls.substr(3);
	   int strategy = 6;
	   sw->setStrategy(strategy);
	   sw->setDelta(0.2);
	   print_str += "Subgrouping Solis Wets";
	   ls = sw;
	}
        else if (arg_ls == "mts1") {
	    ls = new MTSLS1(0.4, 1e-15);
	    print_str += "MTS1 Local Search\nMTS::maxsigma : 0.4\nMTS::minsigma : 1E-15";
	}
	else if (arg_ls == "mts2") {
	    ls = new MTSLS2(0.4, 1e-15);
	    print_str += "MTS2 Local Search\nMTS::maxsigma : 0.4\nMTS::minsigma : 1E-15";
	}
	else if (arg_ls == "simplex") {
	    ls = new SimplexDim();
	    print_str += "SimplexDim";
	}
	else if (arg_ls == "cmaes") {
	   CMAESHansen *cmaes = new CMAESHansen("cmaesinit.par");
	   cmaes->searchNeighborhood(0.5);
	   ls = cmaes;
	   print_str += "CMAESHansen\nCMAES::Neighborhood: 0.5";
	}
	else {
		throw string("localsearch '" +arg_ls +"' is unknown");
	}

    if (debug) {
     printf("%s\n", print_str.c_str());
    }

     return ls;
}

void set_Effort(Hybrid *hybrid, string effort) {
    double ratio;

    if (effort != "") {
       ratio = string_to_double(effort);
       assert(ratio > 0 && ratio < 1);
    }
    else {
       ratio = 0.5;
    }

    if (debug) {
     printf("LS::Effort: %f\n", ratio);
    }
     hybrid->setEffortRatio(ratio);
}

void set_MaxEval(IEA *ea, string maxeval) {
   if (maxeval != "") {
      int imaxeval = atoi(maxeval.c_str());
      assert(imaxeval > 0);
      unsigned maxeval = imaxeval; 

      if (debug) {
      printf("EA::MaxEval: %u\n", maxeval);
      }
      ea->setMaxEval(maxeval);

   }

}
