/**
 * Copyright 2008, Daniel Molina Cabrera <danimolina@gmail.com>
 * 
 * This file is part of software Realea
 * 
 * Realea is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Realea is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <realea/problem/problemsoft.h>
#include <realea/common/ilocalsearch.h>
#include <realea/common/ea.h>
#include <realea/common/srandom.h>
#include "get_util.h"
#include "malschains.h"
#include <iostream>
#include <cstdio>
#include <cstdlib>
#include <cassert>
#include <sstream>

using namespace realea;
using namespace std;
using std::auto_ptr;

static unsigned int dim = 1000;

void usage(char *prog) {
	string eas = get_EANames("|");
	string lses = get_LSNames("|");
	printf("This algorithm applies the MA-LS-Chains algorithm to the Special Issue of SoftComputing for Large Scale Problems.\n");
	printf("usage: %s fun dim seed maxrun\n", prog, lses.c_str());
	printf("Where\tfun is the number of function (1 to 19)\n\tdim is the dimensionality (50,100,200,500,1000)\n");
	printf("\tseed is the seed id (number between 1 and 25)\n\tmaxrun is the maximum times (default=25)\n");
}

Hybrid *processData(string arg_ls, DomainReal *domain, Random &random) {
     string arg_ratio,arg_maxeval;

     string alg = "ssga_blx-0.5";
//     alg = "ssga";
     IEA *ea = get_EA(alg, random);
     ILocalSearch *ls = get_LS(arg_ls, domain, &random);
     MALSChains *ma = new MALSChains(ea, ls);
//     ma->setDebug();
     ma->setRestart(new RestartBest());
     ma->setPopsize(100);
//     ma->setThreshold(1e-8,3);
     Hybrid *hybrid = ma;
     set_Effort(hybrid, "0.8");
     hybrid->setIntensity(500);
     set_MaxEval(hybrid, arg_maxeval);
     return hybrid;
}

double string_to_double( const std::string& s )
{
   std::istringstream i(s);
   double x;
   if (!(i >> x))
     return 0;
   return x;
} 


int main(int argc, char **argv) {
    setbuf(stdout, NULL);
    unsigned long seeds[] = {12345679, 325974, 343593, 97063, 1274389, 3967303, 9073453, 56978025, 26978325, 3283043423, 
			    532875962, 5235907, 628, 23690, 2358032, 1654656072, 296284734, 1111510520, 95621335, 1689121,
			    2441276861u, 3957613033u, 2269123107u, 1217131708, 1895407606};
    int maxseed = 25;

    try {
        set_InitVerbose();
	int fun;

	if (argc < 5) {
	    usage(argv[0]);
	    exit(0);
	}

	fun = atoi(argv[1]);
	dim = atoi(argv[2]);

	if (dim > 1000) {
	    dim = 1000;
	}

	string arg_ls = "ssw";
	int seed_id = atoi(argv[3]);
	if (seed_id <= 0 || seed_id > maxseed) {
           usage(argv[0]);
	   exit(0);
	}

	unsigned long seed = seeds[seed_id-1]; 

	int maxRun = atoi(argv[4]);

	printf("Fun[SoftComputing]: %d[%d]\n", fun, dim);
	printf("Seed: %ld\n", seed);
	Random random(new SRandom(seed)); 

	tChromosomeReal sol(dim);
	tFitness fitness=0;

	ProblemSoftComputing problem(dim);
	ProblemPtr prob = problem.get(fun);
	
	Hybrid *hybrid = processData(arg_ls, prob->getDomain(), random);	
	EA alg(hybrid, prob);

	tFitness sum=0;

	// Defino el problema indicando el umbral 
	for (int num = 1; num <= maxRun; num++) {
	    alg.apply(sol, &fitness);
	    printf("%Le\n", fitness);
	    sum += fitness;
	}
	printf("mean:%Le\n", sum/maxRun);

    } catch (ConfigException *e) {
	    cerr <<"ConfigException : " <<e->what() <<endl;
	    delete e;
    } catch (RunningException *e) {
	    cerr <<"RunningException : " <<e->what() <<endl;
	    delete e;
    } catch (runtime_error *e) {
	    cerr <<"runtime_error: " <<e->what() <<endl;
	    delete e;
    } catch (exception &e) {
	    cerr <<"Exception : " <<e.what() <<endl;
    }
    catch (string &str) {
	    cerr <<"StrException : " <<str <<endl;
    }
    catch (string *str) {
	    cerr <<"StrException : " <<*str <<endl;
	    delete str;
    }

    return 0;
}
